"""
`create_run_test.py`
=======================================================================
Unit tests for code to create CI run records based on CI environment variables
* Author(s): Bryan Siepert
"""
from uuid import uuid1
import pytest
from unittest.mock import MagicMixin, MagicMock, call
from embedops_cli.eotools.ci_run import CIRun, create_entry

from embedops_cli.api.models import CIRun as APICIRun
from embedops_cli.api import rest

"""
- Given an
    - analysis type
    - commit sha
    - branch name
    - embedops api key
    - embedops host
- Then
    - The commit sha, branch name, and analysis type should be set as parameters to the CI run creation API call
    - An API Call should be made to the CI run creation endpoint on the given host
- Given
    - An API response with a 2xx return code
- Then
    - The included record id should be printed to stdout
    - The return code should be 0
"""


class TestEntrypoint:
    @pytest.fixture(autouse=True)
    def configure_env(self, monkeypatch, mocker):
        monkeypatch.setenv("CI", "true")
        monkeypatch.setenv("EMBEDOPS_API_REPO_KEY", "special_pwefix")
        monkeypatch.setenv("EMBEDOPS_ANALYSIS_TYPE", "memusage")
        monkeypatch.setenv("BITBUCKET_BRANCH", "main")
        monkeypatch.setenv("BITBUCKET_BUILD_NUMBER", "10")
        monkeypatch.setenv("BITBUCKET_COMMIT", "abcd1234")
        monkeypatch.setenv(
            "BITBUCKET_GIT_HTTP_ORIGIN", "http://bitbucket.org/dojofive/embedops-tools"
        )
        monkeypatch.setenv("BITBUCKET_STEP_UUID", "yoo-yoo-eye-dee")
        self.ci_run_caller = mocker.patch(
            "embedops_cli.eotools.create_run.CIRun.create"
        )
        self.logger_mock = mocker.patch("embedops_cli.eotools.ci_run._logger")
        self.ci_run = CIRun()
        yield
        del self.ci_run

    def test_ci_envvar_not_set_is_local(self, monkeypatch):
        """If we're not in CI we shant create a CI run, return zero, and output "LOCAL" for the id"""

        monkeypatch.delenv("CI")

        with pytest.raises(SystemExit) as pytest_wrapped_e:
            create_entry()
        assert pytest_wrapped_e.type == SystemExit
        assert pytest_wrapped_e.value.code == 0

        self.logger_mock.info.assert_called_with("LOCAL")
        self.ci_run_caller.assert_not_called()

    def test_no_commit_id(self, monkeypatch):
        if getenv("BITBUCKET_COMMIT"):
            monkeypatch.delenv("BITBUCKET_COMMIT")
        if getenv("GITHUB_SHA"):
            monkeypatch.delenv("GITHUB_SHA")
        if getenv("CI_COMMIT_SHA"):
            monkeypatch.delenv("CI_COMMIT_SHA")
        ci_run = CIRun()
        with pytest.raises(SystemExit) as pytest_wrapped_e:
            ci_run.create_main()
        assert pytest_wrapped_e.type == SystemExit
        assert pytest_wrapped_e.value.code == 1

        self.logger_mock.error.has_calls(call("ERROR: No commit id envvar found"))
        self.ci_run_caller.assert_not_called()

    def test_no_branch_name(self, monkeypatch):
        monkeypatch.delenv("BITBUCKET_BRANCH")
        ci_run = CIRun()
        with pytest.raises(SystemExit) as pytest_wrapped_e:
            ci_run.create_main()
        assert pytest_wrapped_e.type == SystemExit
        assert pytest_wrapped_e.value.code == 1

        self.logger_mock.error.assert_has_calls(
            [call("ERROR: No branch name envvar found")]
        )
        self.ci_run_caller.assert_not_called()

    def test_no_analysis_type(self, monkeypatch):
        monkeypatch.delenv("EMBEDOPS_ANALYSIS_TYPE")
        ci_run = CIRun()
        with pytest.raises(SystemExit) as pytest_wrapped_e:
            ci_run.create_main()
        assert pytest_wrapped_e.type == SystemExit
        assert pytest_wrapped_e.value.code == 1

        self.logger_mock.error.assert_has_calls(
            [call("ERROR: No analysis type envvar found")]
        )
        self.ci_run_caller.assert_not_called()

    def test_multiple_errors_shown(self, monkeypatch):

        monkeypatch.delenv("EMBEDOPS_ANALYSIS_TYPE", "memusage")
        # remove commit envvars
        if getenv("BITBUCKET_COMMIT"):
            monkeypatch.delenv("BITBUCKET_COMMIT")
        if getenv("GITHUB_SHA"):
            monkeypatch.delenv("GITHUB_SHA")
        if getenv("CI_COMMIT_SHA"):
            monkeypatch.delenv("CI_COMMIT_SHA")

        # remove branch envvars
        if getenv("BITBUCKET_BRANCH"):
            monkeypatch.delenv("BITBUCKET_BRANCH")
        if getenv("GITHUB_REF_NAME"):
            monkeypatch.delenv("GITHUB_REF_NAME")
        if getenv("CI_COMMIT_REF_NAME"):
            monkeypatch.delenv("CI_COMMIT_REF_NAME")

        ci_run = CIRun()
        with pytest.raises(SystemExit) as pytest_wrapped_e:
            ci_run.create_main()
        assert pytest_wrapped_e.type == SystemExit
        assert pytest_wrapped_e.value.code == 1

        expected_calls = [
            call("ERROR: No analysis type envvar found"),
            call("ERROR: No branch name envvar found"),
            call("ERROR: No commit id envvar found"),
        ]

        self.logger_mock.error.assert_has_calls(expected_calls, any_order=True)
        self.ci_run_caller.assert_not_called()

    def test_missing_api_key(self, monkeypatch):
        monkeypatch.delenv("EMBEDOPS_API_REPO_KEY")
        ci_run = CIRun()
        with pytest.raises(SystemExit) as pytest_wrapped_e:
            ci_run.create_main()
        assert pytest_wrapped_e.type == SystemExit
        assert pytest_wrapped_e.value.code == 1

        self.logger_mock.error.assert_called_with(
            "ERROR: No EMBEDOPS_API_REPO_KEY envvar found"
        )

    def test_successful_call(self):
        expected_id = "yabbazabba"
        new_ci_run = MagicMock()
        new_ci_run.id = expected_id
        self.ci_run_caller.return_value = new_ci_run
        with pytest.raises(SystemExit) as pytest_wrapped_e:
            create_entry()
        assert pytest_wrapped_e.type == SystemExit
        assert pytest_wrapped_e.value.code == 0

        self.ci_run_caller.assert_called_with("memusage", "abcd1234", "main", None, None)
        self.logger_mock.info.assert_called_with(expected_id)

    def test_successful_call_with_job_name_and_pipeline_id(self, monkeypatch):
        monkeypatch.setenv("EMBEDOPS_JOB_NAME", "build_job")
        monkeypatch.setenv("BITBUCKET_PIPELINE_UUID", "pipe-1234")
        expected_id = "yabbazabba"
        new_ci_run = MagicMock()
        new_ci_run.id = expected_id
        self.ci_run_caller.return_value = new_ci_run
        with pytest.raises(SystemExit) as pytest_wrapped_e:
            create_entry()
        assert pytest_wrapped_e.type == SystemExit
        assert pytest_wrapped_e.value.code == 0

        self.ci_run_caller.assert_called_with(
            "memusage", "abcd1234", "main", "pipe-1234", "build_job"
        )
        self.logger_mock.info.assert_called_with(expected_id)


from os import getenv
import importlib


class TestSwaggerCall:
    @pytest.fixture(autouse=True)
    def configure_env(self, monkeypatch, mocker):
        self.expected_host = "https://dev-01.embedops.io"
        self.expected_type = "memusage"
        self.expected_commit_id = "deadb00f"
        self.expected_branch = "main"
        monkeypatch.setenv("CI", "true")
        monkeypatch.setenv("EMBEDOPS_HOST", self.expected_host)
        monkeypatch.setenv("EMBEDOPS_API_REPO_KEY", "special_pwefix")
        monkeypatch.setenv("EMBEDOPS_ANALYSIS_TYPE", self.expected_type)
        monkeypatch.setenv("BITBUCKET_COMMIT", self.expected_commit_id)
        monkeypatch.setenv("BITBUCKET_BRANCH", self.expected_branch)
        monkeypatch.setenv(
            "BITBUCKET_GIT_HTTP_ORIGIN", "http://bitbucket.org/dojofive/embedops-tools"
        )
        monkeypatch.setenv("BITBUCKET_BUILD_NUMBER", "10")
        monkeypatch.setenv("BITBUCKET_STEP_UUID", "yoo-yoo-eye-de")
        self.logger_mock = mocker.patch("embedops_cli.eotools.ci_run._logger")

        self.ci_run = CIRun()
        yield
        del self.ci_run

    def test_successful_create_response(self, mocker):
        mocked_id = "123123"
        ci_run = MagicMock()
        ci_run.id = mocked_id

        get_client = mocker.patch("embedops_cli.eotools.ci_run.get_client")

        get_client().create_ci_run_from_ci.return_value = ci_run

        response = self.ci_run.create("memusage", "deadb00f", "main")
        assert response.id == mocked_id

    def test_failed_create_response(self, mocker):

        get_client = mocker.patch("embedops_cli.eotools.ci_run.get_client")

        get_client().create_ci_run_from_ci.side_effect = ValueError("garbage!")
        with pytest.raises(SystemExit) as pytest_wrapped_e:
            create_entry()
        self.logger_mock.error.assert_called_with("garbage!")

        get_client().create_ci_run_from_ci.side_effect = TypeError("trash!")
        with pytest.raises(SystemExit) as pytest_wrapped_e:
            create_entry()
        self.logger_mock.error.assert_called_with("trash!")

        get_client().create_ci_run_from_ci.side_effect = rest.ApiException("refuse!")
        with pytest.raises(SystemExit) as pytest_wrapped_e:
            create_entry()
        self.logger_mock.error.assert_called_with("refuse!")

        assert pytest_wrapped_e.type == SystemExit
        assert pytest_wrapped_e.value.code == 1



# coding: utf-8

"""
    EmbedOps API

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: 1.0.0
    Contact: support@embedops.com
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six


class Project(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        "name": "str",
        "created_at": "datetime",
        "id": "str",
        "org_id": "str",
        "repo_ids": "list[str]",
        "updated_at": "datetime",
    }

    attribute_map = {
        "name": "name",
        "created_at": "createdAt",
        "id": "id",
        "org_id": "orgId",
        "repo_ids": "repoIds",
        "updated_at": "updatedAt",
    }

    def __init__(
        self,
        name=None,
        created_at=None,
        id=None,
        org_id=None,
        repo_ids=None,
        updated_at=None,
    ):  # noqa: E501
        """Project - a model defined in Swagger"""  # noqa: E501
        self._name = None
        self._created_at = None
        self._id = None
        self._org_id = None
        self._repo_ids = None
        self._updated_at = None
        self.discriminator = None
        self.name = name
        self.created_at = created_at
        self.id = id
        self.org_id = org_id
        if repo_ids is not None:
            self.repo_ids = repo_ids
        self.updated_at = updated_at

    @property
    def name(self):
        """Gets the name of this Project.  # noqa: E501


        :return: The name of this Project.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this Project.


        :param name: The name of this Project.  # noqa: E501
        :type: str
        """
        if name is None:
            raise ValueError(
                "Invalid value for `name`, must not be `None`"
            )  # noqa: E501

        self._name = name

    @property
    def created_at(self):
        """Gets the created_at of this Project.  # noqa: E501


        :return: The created_at of this Project.  # noqa: E501
        :rtype: datetime
        """
        return self._created_at

    @created_at.setter
    def created_at(self, created_at):
        """Sets the created_at of this Project.


        :param created_at: The created_at of this Project.  # noqa: E501
        :type: datetime
        """
        if created_at is None:
            raise ValueError(
                "Invalid value for `created_at`, must not be `None`"
            )  # noqa: E501

        self._created_at = created_at

    @property
    def id(self):
        """Gets the id of this Project.  # noqa: E501


        :return: The id of this Project.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this Project.


        :param id: The id of this Project.  # noqa: E501
        :type: str
        """
        if id is None:
            raise ValueError("Invalid value for `id`, must not be `None`")  # noqa: E501

        self._id = id

    @property
    def org_id(self):
        """Gets the org_id of this Project.  # noqa: E501


        :return: The org_id of this Project.  # noqa: E501
        :rtype: str
        """
        return self._org_id

    @org_id.setter
    def org_id(self, org_id):
        """Sets the org_id of this Project.


        :param org_id: The org_id of this Project.  # noqa: E501
        :type: str
        """
        if org_id is None:
            raise ValueError(
                "Invalid value for `org_id`, must not be `None`"
            )  # noqa: E501

        self._org_id = org_id

    @property
    def repo_ids(self):
        """Gets the repo_ids of this Project.  # noqa: E501


        :return: The repo_ids of this Project.  # noqa: E501
        :rtype: list[str]
        """
        return self._repo_ids

    @repo_ids.setter
    def repo_ids(self, repo_ids):
        """Sets the repo_ids of this Project.


        :param repo_ids: The repo_ids of this Project.  # noqa: E501
        :type: list[str]
        """

        self._repo_ids = repo_ids

    @property
    def updated_at(self):
        """Gets the updated_at of this Project.  # noqa: E501


        :return: The updated_at of this Project.  # noqa: E501
        :rtype: datetime
        """
        return self._updated_at

    @updated_at.setter
    def updated_at(self, updated_at):
        """Sets the updated_at of this Project.


        :param updated_at: The updated_at of this Project.  # noqa: E501
        :type: datetime
        """
        if updated_at is None:
            raise ValueError(
                "Invalid value for `updated_at`, must not be `None`"
            )  # noqa: E501

        self._updated_at = updated_at

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(
                    map(lambda x: x.to_dict() if hasattr(x, "to_dict") else x, value)
                )
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict())
                        if hasattr(item[1], "to_dict")
                        else item,
                        value.items(),
                    )
                )
            else:
                result[attr] = value
        if issubclass(Project, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, Project):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

# flake8: noqa

"""
    EmbedOps API

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: 1.0.0
    Contact: support@embedops.com
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

from __future__ import absolute_import

# import apis into sdk package
from embedops_cli.api.api.default_api import DefaultApi

# import ApiClient
from embedops_cli.api.api_client import ApiClient
from embedops_cli.api.configuration import Configuration

# import models into sdk package
from embedops_cli.api.models.aws_access_key import AwsAccessKey
from embedops_cli.api.models.aws_access_key_access_key import AwsAccessKeyAccessKey
from embedops_cli.api.models.branch_names import BranchNames
from embedops_cli.api.models.ci_provider import CIProvider
from embedops_cli.api.models.ci_run import CIRun
from embedops_cli.api.models.ci_run_artifact import CIRunArtifact
from embedops_cli.api.models.ci_run_artifact_create_props import (
    CIRunArtifactCreateProps,
)
from embedops_cli.api.models.ci_run_create_props import CIRunCreateProps
from embedops_cli.api.models.ci_run_update_props import CIRunUpdateProps
from embedops_cli.api.models.ci_o_auth_token_update_props import CiOAuthTokenUpdateProps
from embedops_cli.api.models.ci_o_auth_tokens import CiOAuthTokens
from embedops_cli.api.models.ci_run_id_artifacts_body import CiRunIdArtifactsBody
from embedops_cli.api.models.created_at import CreatedAt
from embedops_cli.api.models.gitlab_access_token import GitlabAccessToken
from embedops_cli.api.models.gitlab_access_token_token import GitlabAccessTokenToken
from embedops_cli.api.models.group import Group
from embedops_cli.api.models.group_create_props import GroupCreateProps
from embedops_cli.api.models.group_update_props import GroupUpdateProps
from embedops_cli.api.models.inline_response400 import InlineResponse400
from embedops_cli.api.models.inline_response404 import InlineResponse404
from embedops_cli.api.models.job_def import JobDef
from embedops_cli.api.models.job_def_create_props import JobDefCreateProps
from embedops_cli.api.models.job_def_update_props import JobDefUpdateProps
from embedops_cli.api.models.job_template import JobTemplate
from embedops_cli.api.models.job_template_update_props import JobTemplateUpdateProps
from embedops_cli.api.models.metric import Metric
from embedops_cli.api.models.metric_create_props import MetricCreateProps
from embedops_cli.api.models.metric_result import MetricResult
from embedops_cli.api.models.metric_result_ci_run import MetricResultCiRun
from embedops_cli.api.models.metric_result_ci_run_repo import MetricResultCiRunRepo
from embedops_cli.api.models.metric_result_ci_run_repo_project import (
    MetricResultCiRunRepoProject,
)
from embedops_cli.api.models.metric_result_ci_run_repo_project_org import (
    MetricResultCiRunRepoProjectOrg,
)
from embedops_cli.api.models.myself_clitelemetry_body import MyselfClitelemetryBody
from embedops_cli.api.models.org import Org
from embedops_cli.api.models.org_create_props import OrgCreateProps
from embedops_cli.api.models.org_update_props import OrgUpdateProps
from embedops_cli.api.models.project import Project
from embedops_cli.api.models.project_create_props import ProjectCreateProps
from embedops_cli.api.models.project_group_create_props import ProjectGroupCreateProps
from embedops_cli.api.models.project_group_update_props import ProjectGroupUpdateProps
from embedops_cli.api.models.project_update_props import ProjectUpdateProps
from embedops_cli.api.models.repo import Repo
from embedops_cli.api.models.repo_api_key import RepoApiKey
from embedops_cli.api.models.repo_create_props import RepoCreateProps
from embedops_cli.api.models.repo_pipeline_config import RepoPipelineConfig
from embedops_cli.api.models.repo_pipeline_config_create_props import (
    RepoPipelineConfigCreateProps,
)
from embedops_cli.api.models.repo_pipeline_config_update_props import (
    RepoPipelineConfigUpdateProps,
)
from embedops_cli.api.models.repo_pipeline_config_update_props_jobs import (
    RepoPipelineConfigUpdatePropsJobs,
)
from embedops_cli.api.models.repo_update_props import RepoUpdateProps
from embedops_cli.api.models.user import User
from embedops_cli.api.models.user_group_invite_props import UserGroupInviteProps
from embedops_cli.api.models.user_group_props import UserGroupProps
from embedops_cli.api.models.user_group_props_group import UserGroupPropsGroup
from embedops_cli.api.models.user_group_update_props import UserGroupUpdateProps
from embedops_cli.api.models.user_update_props import UserUpdateProps
from embedops_cli.config import settings


def get_client():
    api_host = settings.get("host")
    api_key = settings.get("api_repo_key", None)

    configuration = Configuration()
    configuration.api_key["X-API-Key"] = api_key
    configuration.host = f"{api_host}/api/v1"
    api_client = ApiClient(configuration=configuration)

    return DefaultApi(api_client=api_client)
